@@ -4,6 +4,7 @@
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MagicHash             #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE PackageImports        #-}
 {-# LANGUAGE PolyKinds             #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE Trustworthy           #-}
@@ -87,7 +88,6 @@ import Foreign.Storable       (alignment, sizeOf)
 import GHC.Base               (ByteArray#)
 import GHC.Conc               (ThreadId (..))
 import GHC.Fingerprint.Type   (Fingerprint (..))
-import GHC.Prim               (ThreadId#)
 import GHC.Word               (Word (..))
 import System.IO.Unsafe       (unsafeDupablePerformIO)
 import System.Mem.StableName  (StableName, hashStableName)
@@ -116,6 +116,10 @@ import qualified Data.Tree                      as Tree
 
 import GHC.Generics
 
+#if MIN_VERSION_base(4,19,0)
+import GHC.Conc.Sync (fromThreadId)
+#else
+import GHC.Prim (ThreadId#)
 #if __GLASGOW_HASKELL__ >= 904
 import Foreign.C.Types (CULLong (..))
 #elif __GLASGOW_HASKELL__ >= 900
@@ -123,6 +127,7 @@ import Foreign.C.Types (CLong (..))
 #else
 import Foreign.C.Types (CInt (..))
 #endif
+#endif
 
 #ifdef VERSION_ghc_bignum
 import GHC.Exts        (Int (..), sizeofByteArray#)
@@ -163,7 +168,25 @@ import Data.Tuple (Solo (..))
 import GHC.Tuple (Solo (..))
 #endif
 
-#if MIN_VERSION_filepath(1,4,100)
+-- filepath >=1.4.100 && <1.5 has System.OsString.Internal.Types module
+#if MIN_VERSION_filepath(1,4,100) && !(MIN_VERSION_filepath(1,5,0))
+#define HAS_OS_STRING_filepath 1
+#else
+#define HAS_OS_STRING_filepath 0
+#endif
+
+-- if we depend on os_string module, then it has System.OsString.Internal.Types
+-- module as well
+#ifdef MIN_VERSION_os_string
+#define HAS_OS_STRING_os_string 1
+#else
+#define HAS_OS_STRING_os_string 0
+#endif
+
+#if HAS_OS_STRING_filepath && HAS_OS_STRING_os_string
+import "os-string" System.OsString.Internal.Types (OsString (..), PosixString (..), WindowsString (..))
+import qualified "filepath" System.OsString.Internal.Types as FP (OsString (..), PosixString (..), WindowsString (..))
+#elif HAS_OS_STRING_filepath || HAS_OS_STRING_os_string
 import System.OsString.Internal.Types (OsString (..), PosixString (..), WindowsString (..))
 #endif
 
@@ -662,7 +685,7 @@ instance Hashable BSI.ShortByteString where
     hashWithSalt salt sbs@(BSI.SBS ba) =
         hashByteArrayWithSalt ba 0 (BSI.length sbs) (hashWithSalt salt (BSI.length sbs))
 
-#if MIN_VERSION_filepath(1,4,100)
+#if HAS_OS_STRING_filepath || HAS_OS_STRING_os_string
 -- | @since 1.4.2.0
 instance Hashable PosixString where
     hashWithSalt salt (PosixString s) = hashWithSalt salt s
@@ -676,6 +699,17 @@ instance Hashable OsString where
     hashWithSalt salt (OsString s) = hashWithSalt salt s
 #endif
 
+#if HAS_OS_STRING_filepath && HAS_OS_STRING_os_string
+instance Hashable FP.PosixString where
+    hashWithSalt salt (FP.PosixString s) = hashWithSalt salt s
+
+instance Hashable FP.WindowsString where
+    hashWithSalt salt (FP.WindowsString s) = hashWithSalt salt s
+
+instance Hashable FP.OsString where
+    hashWithSalt salt (FP.OsString s) = hashWithSalt salt s
+#endif
+
 #if MIN_VERSION_text(2,0,0)
 
 instance Hashable T.Text where
@@ -707,9 +741,9 @@ instance Hashable TL.Text where
 
 #endif
 
--- | Compute the hash of a ThreadId.
-hashThreadId :: ThreadId -> Int
-hashThreadId (ThreadId t) = hash (fromIntegral (getThreadId t) :: Int)
+#if !MIN_VERSION_base(4,19,0)
+fromThreadId :: ThreadId -> Word64
+fromThreadId (ThreadId t) = fromIntegral (getThreadId t)
 
 -- this cannot be capi, as GHC panics.
 foreign import ccall unsafe "rts_getThreadId" getThreadId
@@ -722,9 +756,10 @@ foreign import ccall unsafe "rts_getThreadId" getThreadId
 #else
     :: ThreadId# -> CInt
 #endif
+#endif
 
 instance Hashable ThreadId where
-    hash = hashThreadId
+    hash = hash . fromThreadId
     hashWithSalt = defaultHashWithSalt
 
 instance Hashable (Ptr a) where
